==> anonnet-0.0.1-20010728.tgz: anonnet/src/crypt.c <==
/* ==========================================================================
 * AnonNet - Anonymous Network Project
 * --------------------------------------------------------------------------
 * Copyright (C) 2001  William Ahern
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
 * ==========================================================================
 */

#include <config.h>

#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <fcntl.h>
#include <mhash.h>

#ifdef HAVE_GMP_H
#include <gmp.h>
#else
#error No gmp.h!
#endif /* HAVE_GMP_H */

#include "common.h"
#include "conf.h"
#include "log.h"
#include "object.h"
#include "bignum.h"
#include "crypt.h"
#include "sha1.h"
#include "tiger.h"


void rand_init() {
	extern object_t	__rand_mutex;
	extern int	__rand_fd;
	extern conf_t	conf;
	
	object_init(&__rand_mutex,NULL,NULL);


	object_lock(&__rand_mutex);
	object_lock(&conf);

	if ((__rand_fd = open(RAND_PATH,O_RDONLY)) < 0) {
		ERROR("could not open random source %s",RAND_PATH);
	}

	object_unlock(&conf);
	object_unlock(&__rand_mutex);

	return; /* void */
}

void rand_get(unsigned char *buf, int bytes) {
	extern object_t	__rand_mutex;
	extern int	__rand_fd;
	int		in;

	object_lock(&__rand_mutex);
	
	while (bytes > 0) {
		if ((in = read(__rand_fd,(void *)buf,bytes)) <= 0) {
			WARN("EOF or error reading from random source %s",RAND_PATH);
			break;
		}
		bytes	-= in;
		buf	+= in;
	}

	object_unlock(&__rand_mutex);

	return; /* void */
}


int hash_get_digest_size(int type) {

	switch (type) {
		case SHA1_160:
			return SHA1_160_DIGEST_SIZE;
		case TIGER_192:
			return TIGER_192_DIGEST_SIZE;
		case TIGER_160:
			return TIGER_160_DIGEST_SIZE;
		case TIGER_128:
			return TIGER_128_DIGEST_SIZE;
		default:
			return 0;
	}

}


int hash_get_block_size(int type) {
	/* both SHA1 and Tiger operate in 64 byte/512 bit blocks */
	return 64;
}


void hash_init(int type,hash_t *hash) {
	hash->type	= type;
	
	switch (type) {
		case SHA1_160:
			sha_init((SHA_CTX *)hash->ctx);
			hash->block_size	= SHA1_160_BLOCK_SIZE;
			hash->digest_size	= SHA1_160_DIGEST_SIZE;
			break;
		case TIGER_192:
			tiger_init((TIGER_CTX *)hash->ctx);
			hash->block_size	= TIGER_192_BLOCK_SIZE;
			hash->digest_size	= TIGER_192_DIGEST_SIZE;
			break;
		case TIGER_160:
			tiger_init((TIGER_CTX *)hash->ctx);
			hash->block_size	= TIGER_160_BLOCK_SIZE;
			hash->digest_size	= TIGER_160_DIGEST_SIZE;
			break;
		case TIGER_128:
			tiger_init((TIGER_CTX *)hash->ctx);
			hash->block_size	= TIGER_128_BLOCK_SIZE;
			hash->digest_size	= TIGER_128_DIGEST_SIZE;
			break;
		default:
			ERROR("Bad hash type requested in hash_init.");
	}	

	return; /* void */

} /* end hash_init */


void hash_update(hash_t *hash,void *plaintext, int size) {
	
	switch (hash->type) {
		case SHA1_160:
			sha_update((SHA_CTX *)hash->ctx,plaintext,size);
			break;
		case TIGER_192:
		case TIGER_160:
		case TIGER_128:
			tiger_update((TIGER_CTX *)hash->ctx,plaintext,size);
			break;
		default:
			ERROR("Some huge error in hash_update!");
	}
	
	return; /* void */
	
} /* end hash_update */


void hash_final(hash_t *hash,void *digest) {

	switch (hash->type) {
		case SHA1_160:
			sha_final((SHA_CTX *)hash->ctx);
			sha_digest((SHA_CTX *)hash->ctx,digest);
			break;
		case TIGER_192:
			tiger_final((TIGER_CTX *)hash->ctx);
			tiger_digest((TIGER_CTX *)hash->ctx,digest);
			break;
		case TIGER_160:
			tiger_final((TIGER_CTX *)hash->ctx);
			tiger160_digest((TIGER_CTX *)hash->ctx,digest);
			break;
		case TIGER_128:
			tiger_final((TIGER_CTX *)hash->ctx);
			tiger128_digest((TIGER_CTX *)hash->ctx,digest);
			break;
		default:
			ERROR("Some huge error in hash_final!");
	}
	
	return; /* void */

} /* hash_final */


void hmac_init(int type, hash_t *hash,void *key, int size) {
	int i;

	hash_init(type,hash);
	
	if (size > hash->block_size) {
		HASH(type,hash->key,key,size);
		hash->key_size = hash->digest_size;
	} else {
		/*bzero(hash->key,sizeof(hash->key));*/
		memcpy(hash->key,key,size);
		hash->key_size = size;
	}


	for (i = 0; i < hash->key_size; i++) {
		hash->ipad[i] = IPAD ^ hash->key[i];
	}

	for (; i < hash->block_size; i++) {
		hash->ipad[i] = IPAD;
	}


	hash_update(hash,hash->ipad,hash->block_size);

	return; /* void */

} /* end hmac_init */


#define hmac_update(hash,plaintext,size) \
	hash_update((hash),(plaintext),(size))


void hmac_final(hash_t *hash,void *digest) {
	int i;
	hash_t pad;
	
	for (i = 0; i < hash->key_size; i++) {
		hash->opad[i] = OPAD ^ hash->key[i];
	}

	for (; i < hash->block_size; i++) {
		hash->opad[i] = OPAD;
	}

	hash_init(hash->type,&pad);
	hash_update(&pad,hash->opad,hash->block_size);
	
	switch (hash->type) {
		case SHA1_160:
			sha_final((SHA_CTX *)hash->ctx);
			sha_digest((SHA_CTX *)hash->ctx,digest);
			break;
		case TIGER_192:
			tiger_final((TIGER_CTX *)hash->ctx);
			tiger_digest((TIGER_CTX *)hash->ctx,digest);
			break;
		case TIGER_160:
			tiger_final((TIGER_CTX *)hash->ctx);
			tiger160_digest((TIGER_CTX *)hash->ctx,digest);
			break;
		case TIGER_128:
			tiger_final((TIGER_CTX *)hash->ctx);
			tiger128_digest((TIGER_CTX *)hash->ctx,digest);
			break;
		default:
			ERROR("Some huge error in hmac_final!");
	}

	hash_update(&pad,digest,hash->digest_size);
	hash_final(&pad,digest);

	return; /* void */

} /* end hmac_final */



/*
 * unsigned char *HMAC(const EVP_MD *evp_md,
 *                    const void *key, int key_len,
 *                    const unsigned char *d, int n,
 *                    unsigned char *md, unsigned int *md_len);
 */
 
/*
 * From TLS 1.0 Draft RFC (absent the MD5/SHA1 split)
 *
 * PRF(secret, seed) = HMAC_hash(secret, A(1) + seed) +
 *                     HMAC_hash(secret, A(2) + seed) +
 *                     HMAC_hash(secret, A(3) + seed) + ...
 *                                                                 
 * Where + indicates concatenation.
 *                                                                    
 * A() is defined as:
 * A(0) = seed
 * A(i) = HMAC_hash(secret, A(i-1))
 *
 */



void prf_expand(int hashid,unsigned char *out,int out_len,unsigned char *secret,int secret_len,unsigned char *seed,int seed_len) {
	MHASH td;
	unsigned char text[PRF_MAX_SEED_SIZE + MAX(PRF_MAX_SEED_SIZE,PRF_MAX_HASH_SIZE)];
	unsigned char *stream;
	int hash_len,text_len;

	secret_len	= MIN(secret_len,PRF_MAX_SECRET_SIZE);
	seed_len	= MIN(seed_len,PRF_MAX_SEED_SIZE);
	hash_len	= hash_get_digest_size(hashid);

	stream = out;

	/* create initial text using seed + seed */
	memcpy(text,seed,seed_len);
	memcpy(&text[seed_len],seed,seed_len);
	text_len = seed_len * 2;

	do {
		/* hash w/ secret */
		/*MHASH_HMAC(hashid,stream,MIN(out_len,hash_len),secret,secret_len,text,text_len);*/
		HMAC(hashid,stream,secret,secret_len,text,text_len);
		memcpy(text,stream,MIN(out_len,hash_len));
		memcpy(&text[hash_len],seed,seed_len);

		text_len	= hash_len + seed_len;
		stream		+= MIN(out_len,hash_len);
		out_len		-= hash_len;
	} while (out_len > 0);

	return; /* void */
}



/* PRF for key expansion */
/* void prf_expand(unsigned char *out, int out_len, const unsigned char *secret, int secret_len, const unsigned char *seed, int seed_len) {
	unsigned char md[EVP_MAX_MD_SIZE];
	unsigned char m[PRF_MAX_SEED_SIZE + MAX(PRF_MAX_SEED_SIZE,EVP_MAX_MD_SIZE)];
	unsigned char sc[PRF_MAX_SECRET_SIZE];
	unsigned char sd[PRF_MAX_SEED_SIZE];
	unsigned int md_len,m_len;
	int out_bytes = 0;
	unsigned char *prev;

	secret_len = MIN(PRF_MAX_SECRET_SIZE,secret_len);
	memcpy(sc,secret,secret_len);
	seed_len = MIN(PRF_MAX_SEED_SIZE,seed_len);
	memcpy(sd,seed,seed_len);
*/
	/* round 1 */
/*	memcpy(&m[0],sd,seed_len);
	memcpy(&m[seed_len],sd,seed_len);
	m_len = seed_len*2;
	HMAC(EVP_sha1(),sc,secret_len,m,m_len,md,&md_len);

	if (out_len <= md_len) {
		memcpy(out,md,out_len);
		return;
	}
	
	memcpy(out,md,md_len);
	out += md_len;
	out_bytes += md_len;

	do {
		memcpy(m,md,md_len);
		memcpy((m+md_len),sd,seed_len);
		m_len = md_len + seed_len;
		HMAC(EVP_sha1(),sc,secret_len,m,m_len,md,&md_len);
		memcpy(out,md,MIN((out_len-out_bytes),md_len));
		out += md_len;
		out_bytes += md_len;
	} while (out_bytes < out_len);
	
} */


void dh_init(dh_t *dh) {

	mpz_init(dh->g); /* generator	*/
	mpz_init(dh->p); /* prime	*/
	mpz_init(dh->z); /* private key	*/
	mpz_init(dh->y); /* public key	*/
	mpz_init(dh->x); /* secret	*/

	return; /* void */

} /* end dh_init */


void dh_destroy(dh_t *dh) {

	mpz_clear(dh->g); /* generator		*/
	mpz_clear(dh->p); /* prime		*/
	mpz_clear(dh->x); /* private key	*/
	mpz_clear(dh->y); /* public key		*/
	mpz_clear(dh->z); /* secret		*/

	return; /* void */	

} /* end dh_destroy */


/*
 * generate size bits prime
 */
void dh_generate_parameters(dh_t *dh,int size,int generator) {

	bignum_random_prime(dh->p,size);
	mpz_set_ui(dh->g,generator);

	dh->p_size_bits		= mpz_sizeinbase(dh->p,2);
	dh->p_size_bytes	= bignum_format_u_length(dh->p);
	
	if (dh->p_size_bits != size)
		WARN("Requested prime size does not seem to match existing: requested % i bits, have %i.",size,dh->p_size_bits);
	
	return; /* void */

} /* end dh_generate_parameters */


void dh_set_prime(dh_t *dh,mpz_t p) {

	mpz_set(dh->p,p);
	
	dh->p_size_bytes	= bignum_format_u_length(dh->p);
	dh->p_size_bits		= mpz_sizeinbase(dh->p,2);

	return; /* void */

}


void dh_set_generator(dh_t *dh,mpz_t g) {

	mpz_set(dh->g,g);
	
	return; /* void */

}


/*
 * generate public key
 *
 * y = g^x % p
 *
 */
void dh_generate_key(dh_t *dh) {
	unsigned char private[DH_MAX_PRIME_BYTE_SIZE];
	unsigned char less;
	int size;

	/* create secret */

	/*rand_get(&less,sizeof(less));
	size = (dh->p_size_bits-less)/8;
	rand_get(private,size);*/
	
	rand_get(private,dh->p_size_bytes - 1);
	bignum_parse_u(dh->x,dh->p_size_bytes - 1,private);

	mpz_powm(dh->y,dh->g,dh->x,dh->p);

	return; /* void */

} /* end dh_generate_key */


/*
 * generate shared secret
 *
 * z = y^x % p
 *
 */
void dh_compute_key(dh_t *dh,mpz_t y1) {

	mpz_powm(dh->z,y1,dh->x,dh->p);	

	return; /* void */

} /* end dh_compute_key */

==> anonnet-0.0.1-20010729.tgz: anonnet/src/crypt.c <==
/* ==========================================================================
 * AnonNet - Anonymous Network Project
 * --------------------------------------------------------------------------
 * Copyright (C) 2001  William Ahern
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
 * ==========================================================================
 */

#include <config.h>

#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <fcntl.h>
#include <mhash.h>

#ifdef HAVE_GMP_H
#include <gmp.h>
#else
#error No gmp.h!
#endif /* HAVE_GMP_H */

#include "common.h"
#include "conf.h"
#include "log.h"
#include "object.h"
#include "bignum.h"
#include "crypt.h"
#include "sha1.h"
#include "tiger.h"


void rand_init() {
	extern conf_t conf;
	rand_ctx *ctx;

	pthread_once(&__rand_once_control,__rand_once_init);

	object_lock(&conf);
	
	if ((ctx = malloc(sizeof(rand_ctx))) == NULL) {
		ERROR("Could not allocate new random control structure.");
		return; /* void */
	}

	if ((ctx->fd = open(conf.random,O_RDONLY)) < 0) {
		ERROR("Could not open random source %s",conf.random);
		return; /* void */
	}

	DEBUG("File descriptor for %s for this thread: %i",conf.random,ctx->fd);

	pthread_setspecific(__rand_ctx_key,ctx);

	object_unlock(&conf);

	return; /* void */
}


void __rand_once_init() {
	pthread_key_create(&__rand_ctx_key,__rand_thread_ctx_free);
	
	return; /* void */
}


void __rand_thread_ctx_free(void *ctx) {
	if (ctx != NULL) {
		DEBUG("Closing random device file descriptor: %i",((rand_ctx *)ctx)->fd);
		close(((rand_ctx *)ctx)->fd);
		free(ctx);
	} else {
		WARN("Random context structure was null on thread exit.");
	}
	
	return; /* void */
}


rand_ctx *rand_getctx() {
	return (rand_ctx *)pthread_getspecific(__rand_ctx_key);
}


void rand_get(rand_ctx *ctx,void *buf, int bytes) {
	extern conf_t conf;
	unsigned char *__buf;
	int in;

	__buf = buf;
	
	if ((ctx != NULL) && (ctx->fd >= 0)) {

		while (bytes > 0) {
			if ((in = read(ctx->fd,__buf,bytes)) <= 0) {
				object_lock(&conf);
				WARN("EOF or error reading from random source %s.",conf.random);
				object_unlock(&conf);
				break;
			}
			bytes	-= in;
			__buf	+= in;
		}

		
	} else {
		WARN("Could not get random context structure in rand_get.");
	}
	
	return; /* void */
}


int hash_get_digest_size(int type) {

	switch (type) {
		case SHA1_160:
			return SHA1_160_DIGEST_SIZE;
		case TIGER_192:
			return TIGER_192_DIGEST_SIZE;
		case TIGER_160:
			return TIGER_160_DIGEST_SIZE;
		case TIGER_128:
			return TIGER_128_DIGEST_SIZE;
		default:
			return 0;
	}

}


int hash_get_block_size(int type) {
	/* both SHA1 and Tiger operate in 64 byte/512 bit blocks */
	return 64;
}


void hash_init(int type,hash_t *hash) {
	hash->type	= type;
	
	switch (type) {
		case SHA1_160:
			sha_init((SHA_CTX *)hash->ctx);
			hash->block_size	= SHA1_160_BLOCK_SIZE;
			hash->digest_size	= SHA1_160_DIGEST_SIZE;
			break;
		case TIGER_192:
			tiger_init((TIGER_CTX *)hash->ctx);
			hash->block_size	= TIGER_192_BLOCK_SIZE;
			hash->digest_size	= TIGER_192_DIGEST_SIZE;
			break;
		case TIGER_160:
			tiger_init((TIGER_CTX *)hash->ctx);
			hash->block_size	= TIGER_160_BLOCK_SIZE;
			hash->digest_size	= TIGER_160_DIGEST_SIZE;
			break;
		case TIGER_128:
			tiger_init((TIGER_CTX *)hash->ctx);
			hash->block_size	= TIGER_128_BLOCK_SIZE;
			hash->digest_size	= TIGER_128_DIGEST_SIZE;
			break;
		default:
			ERROR("Bad hash type requested in hash_init.");
	}	

	return; /* void */

} /* end hash_init */


void hash_update(hash_t *hash,void *plaintext, int size) {
	
	switch (hash->type) {
		case SHA1_160:
			sha_update((SHA_CTX *)hash->ctx,plaintext,size);
			break;
		case TIGER_192:
		case TIGER_160:
		case TIGER_128:
			tiger_update((TIGER_CTX *)hash->ctx,plaintext,size);
			break;
		default:
			ERROR("Some huge error in hash_update!");
	}
	
	return; /* void */
	
} /* end hash_update */


void hash_final(hash_t *hash,void *digest) {

	switch (hash->type) {
		case SHA1_160:
			sha_final((SHA_CTX *)hash->ctx);
			sha_digest((SHA_CTX *)hash->ctx,digest);
			break;
		case TIGER_192:
			tiger_final((TIGER_CTX *)hash->ctx);
			tiger_digest((TIGER_CTX *)hash->ctx,digest);
			break;
		case TIGER_160:
			tiger_final((TIGER_CTX *)hash->ctx);
			tiger160_digest((TIGER_CTX *)hash->ctx,digest);
			break;
		case TIGER_128:
			tiger_final((TIGER_CTX *)hash->ctx);
			tiger128_digest((TIGER_CTX *)hash->ctx,digest);
			break;
		default:
			ERROR("Some huge error in hash_final!");
	}
	
	return; /* void */

} /* hash_final */


void hmac_init(int type, hash_t *hash,void *key, int size) {
	int i;

	hash_init(type,hash);
	
	if (size > hash->block_size) {
		HASH(type,hash->key,key,size);
		hash->key_size = hash->digest_size;
	} else {
		/*bzero(hash->key,sizeof(hash->key));*/
		memcpy(hash->key,key,size);
		hash->key_size = size;
	}


	for (i = 0; i < hash->key_size; i++) {
		hash->ipad[i] = IPAD ^ hash->key[i];
	}

	for (; i < hash->block_size; i++) {
		hash->ipad[i] = IPAD;
	}


	hash_update(hash,hash->ipad,hash->block_size);

	return; /* void */

} /* end hmac_init */


#define hmac_update(hash,plaintext,size) \
	hash_update((hash),(plaintext),(size))


void hmac_final(hash_t *hash,void *digest) {
	int i;
	hash_t pad;
	
	for (i = 0; i < hash->key_size; i++) {
		hash->opad[i] = OPAD ^ hash->key[i];
	}

	for (; i < hash->block_size; i++) {
		hash->opad[i] = OPAD;
	}

	hash_init(hash->type,&pad);
	hash_update(&pad,hash->opad,hash->block_size);
	
	switch (hash->type) {
		case SHA1_160:
			sha_final((SHA_CTX *)hash->ctx);
			sha_digest((SHA_CTX *)hash->ctx,digest);
			break;
		case TIGER_192:
			tiger_final((TIGER_CTX *)hash->ctx);
			tiger_digest((TIGER_CTX *)hash->ctx,digest);
			break;
		case TIGER_160:
			tiger_final((TIGER_CTX *)hash->ctx);
			tiger160_digest((TIGER_CTX *)hash->ctx,digest);
			break;
		case TIGER_128:
			tiger_final((TIGER_CTX *)hash->ctx);
			tiger128_digest((TIGER_CTX *)hash->ctx,digest);
			break;
		default:
			ERROR("Some huge error in hmac_final!");
	}

	hash_update(&pad,digest,hash->digest_size);
	hash_final(&pad,digest);

	return; /* void */

} /* end hmac_final */



/*
 * unsigned char *HMAC(const EVP_MD *evp_md,
 *                    const void *key, int key_len,
 *                    const unsigned char *d, int n,
 *                    unsigned char *md, unsigned int *md_len);
 */
 
/*
 * From TLS 1.0 Draft RFC (absent the MD5/SHA1 split)
 *
 * PRF(secret, seed) = HMAC_hash(secret, A(1) + seed) +
 *                     HMAC_hash(secret, A(2) + seed) +
 *                     HMAC_hash(secret, A(3) + seed) + ...
 *                                                                 
 * Where + indicates concatenation.
 *                                                                    
 * A() is defined as:
 * A(0) = seed
 * A(i) = HMAC_hash(secret, A(i-1))
 *
 */



void prf_expand(int hashid,unsigned char *out,int out_len,unsigned char *secret,int secret_len,unsigned char *seed,int seed_len) {
	MHASH td;
	unsigned char text[PRF_MAX_SEED_SIZE + MAX(PRF_MAX_SEED_SIZE,PRF_MAX_HASH_SIZE)];
	unsigned char *stream;
	int hash_len,text_len;

	secret_len	= MIN(secret_len,PRF_MAX_SECRET_SIZE);
	seed_len	= MIN(seed_len,PRF_MAX_SEED_SIZE);
	hash_len	= hash_get_digest_size(hashid);

	stream = out;

	/* create initial text using seed + seed */
	memcpy(text,seed,seed_len);
	memcpy(&text[seed_len],seed,seed_len);
	text_len = seed_len * 2;

	do {
		/* hash w/ secret */
		/*MHASH_HMAC(hashid,stream,MIN(out_len,hash_len),secret,secret_len,text,text_len);*/
		HMAC(hashid,stream,secret,secret_len,text,text_len);
		memcpy(text,stream,MIN(out_len,hash_len));
		memcpy(&text[hash_len],seed,seed_len);

		text_len	= hash_len + seed_len;
		stream		+= MIN(out_len,hash_len);
		out_len		-= hash_len;
	} while (out_len > 0);

	return; /* void */
}



/* PRF for key expansion */
/* void prf_expand(unsigned char *out, int out_len, const unsigned char *secret, int secret_len, const unsigned char *seed, int seed_len) {
	unsigned char md[EVP_MAX_MD_SIZE];
	unsigned char m[PRF_MAX_SEED_SIZE + MAX(PRF_MAX_SEED_SIZE,EVP_MAX_MD_SIZE)];
	unsigned char sc[PRF_MAX_SECRET_SIZE];
	unsigned char sd[PRF_MAX_SEED_SIZE];
	unsigned int md_len,m_len;
	int out_bytes = 0;
	unsigned char *prev;

	secret_len = MIN(PRF_MAX_SECRET_SIZE,secret_len);
	memcpy(sc,secret,secret_len);
	seed_len = MIN(PRF_MAX_SEED_SIZE,seed_len);
	memcpy(sd,seed,seed_len);
*/
	/* round 1 */
/*	memcpy(&m[0],sd,seed_len);
	memcpy(&m[seed_len],sd,seed_len);
	m_len = seed_len*2;
	HMAC(EVP_sha1(),sc,secret_len,m,m_len,md,&md_len);

	if (out_len <= md_len) {
		memcpy(out,md,out_len);
		return;
	}
	
	memcpy(out,md,md_len);
	out += md_len;
	out_bytes += md_len;

	do {
		memcpy(m,md,md_len);
		memcpy((m+md_len),sd,seed_len);
		m_len = md_len + seed_len;
		HMAC(EVP_sha1(),sc,secret_len,m,m_len,md,&md_len);
		memcpy(out,md,MIN((out_len-out_bytes),md_len));
		out += md_len;
		out_bytes += md_len;
	} while (out_bytes < out_len);
	
} */


void dh_init(dh_t *dh) {

	mpz_init(dh->g); /* generator	*/
	mpz_init(dh->p); /* prime	*/
	mpz_init(dh->z); /* private key	*/
	mpz_init(dh->y); /* public key	*/
	mpz_init(dh->x); /* secret	*/

	return; /* void */

} /* end dh_init */


void dh_destroy(dh_t *dh) {

	mpz_clear(dh->g); /* generator		*/
	mpz_clear(dh->p); /* prime		*/
	mpz_clear(dh->x); /* private key	*/
	mpz_clear(dh->y); /* public key		*/
	mpz_clear(dh->z); /* secret		*/

	return; /* void */	

} /* end dh_destroy */


/*
 * generate size bits prime
 */
void dh_generate_parameters(dh_t *dh,int size,int generator) {

	bignum_random_prime(dh->p,size);
	mpz_set_ui(dh->g,generator);

	dh->p_size_bits		= mpz_sizeinbase(dh->p,2);
	dh->p_size_bytes	= bignum_format_u_length(dh->p);
	
	if (dh->p_size_bits != size)
		WARN("Requested prime size does not seem to match existing: requested % i bits, have %i.",size,dh->p_size_bits);
	
	return; /* void */

} /* end dh_generate_parameters */


void dh_set_prime(dh_t *dh,mpz_t p) {

	mpz_set(dh->p,p);
	
	dh->p_size_bytes	= bignum_format_u_length(dh->p);
	dh->p_size_bits		= mpz_sizeinbase(dh->p,2);

	return; /* void */

}


void dh_set_generator(dh_t *dh,mpz_t g) {

	mpz_set(dh->g,g);
	
	return; /* void */

}


/*
 * generate public key
 *
 * y = g^x % p
 *
 */
void dh_generate_key(dh_t *dh) {
	unsigned char private[DH_MAX_PRIME_BYTE_SIZE];
	unsigned char less;
	int size;

	/* create secret */

	/*rand_get(&less,sizeof(less));
	size = (dh->p_size_bits-less)/8;
	rand_get(private,size);*/
	
	rand_get(rand_getctx(),private,dh->p_size_bytes - 1);
	bignum_parse_u(dh->x,dh->p_size_bytes - 1,private);

	mpz_powm(dh->y,dh->g,dh->x,dh->p);

	return; /* void */

} /* end dh_generate_key */


/*
 * generate shared secret
 *
 * z = y^x % p
 *
 */
void dh_compute_key(dh_t *dh,mpz_t y1) {

	mpz_powm(dh->z,y1,dh->x,dh->p);	

	return; /* void */

} /* end dh_compute_key */
